from pymasmovil.client import Client


class Contract():
    """
    Parent class to OrderItem and Asset,
    which share a very similar class structure and attributes
    """

    id = ''
    account_id = ''
    name = ''
    surname = ''
    productName = ''
    phone = ''
    status = ''
    createdDate = ''
    attributes = {
        'Apellidos': '',
        'Fecha_Portabilidad_Saliente': '',
        'ICCID_Donante': '',
        'Nombre': '',
        'Numero_de_Documento': '',
        'Operador_Donante_Movil': '',
        'Operador_Receptor_Movil': '',
        'Tipo_de_Documento': '',
        'Tipo_de_Linea': '',
        'Fecha_de_solicitud_del_abonado': '',
        'Porcentaje_Consumo_Bono': ''
    }
    simAttributes = {
        'ICCID': '',
        'IMSI': '',
        'PIN': '',
        'PIN2': '',
        'PUK': '',
        'PUK2': ''
    }

    def __init__(self, **kwargs):

        for key, value in kwargs.items():
            if key == 'attributes':
                for inner_key, inner_value in value.items():
                    if inner_key in self.attributes:
                        self.attributes[inner_key] = inner_value
            if key == 'simAttributes':
                for inner_key, inner_value in value.items():
                    if inner_key in self.simAttributes:
                        self.simAttributes[inner_key] = inner_value
            else:
                if hasattr(self.__class__, key):
                    setattr(self, key, value)

    @classmethod
    def get(cls, session, contract_id):
        """
            Returns a Contract instance (order_item or asset) obtained by id.
            :param contract_id:

            :return: Contract:
        """

        response = Client(session).get(
            route='{}/{}'.format(cls._route, contract_id))

        return response



from pymasmovil.errors.exceptions import UnknownMMError


class MMError(Exception):

    def __init__(self):
        self.status_code = ''
        self.message = ''
        self.fields = ''

    def build(self, response):
        """
        MM API should return always an HTTPError with this structure:
        {
            "success": false,
            "errors": [
                {
                    "fields": "",
                    "statusCode": "",
                    "message": ""
                }
            ]
        }
        But in at least one endpoint the API returns this dictionary inside a list.
        When the session is invalid the MM API returns also another different list.

        This method builds a MMError from these different structures.
        """

        bad_response = response.json()

        try:
            if isinstance(bad_response, list):
                if 'errorCode' in bad_response[0]:
                    self.status_code = response.status_code
                    self.message = '{}: {}'.format(bad_response[0]['errorCode'],
                                                   bad_response[0]['message'])
                else:
                    self._set_attributes_with_error_dct(bad_response[0]['errors'][0])
            else:
                self._set_attributes_with_error_dct(bad_response['errors'][0])
        except (TypeError, KeyError):
            raise UnknownMMError(bad_response)

    def _set_attributes_with_error_dct(self, error_dct):
        self.status_code = error_dct['statusCode']
        self.message = error_dct['message']
        self.fields = error_dct['fields']



from pymasmovil.client import Client
from pymasmovil.models.order_item import OrderItem
from pymasmovil.models.asset import Asset
from pymasmovil.errors.exceptions import AccountRequiredParamsError


class Account():
    _route = '/v0/accounts'

    town = ""
    surname = ""
    stair = ""
    roadType = ""
    roadNumber = ""
    roadName = ""
    region = ""
    province = ""
    postalCode = ""
    phone = ""
    name = ""
    id = ""
    flat = ""
    email = ""
    door = ""
    documentType = ""
    documentNumber = ""
    corporateName = ""
    buildingPortal = ""

    def __init__(self, **kwargs):
        for key, value in kwargs.items():
            if hasattr(Account, key):
                setattr(self, key, value)

    @classmethod
    def get(cls, session, account_id):
        """
        Returns a account instance obtained by id.

        :param id:
        :return: Account:
        """

        account_response = Client(session).get(
            route='{}/{}'.format(cls._route, account_id))

        return Account(**account_response)

    @classmethod
    def create(cls, session, **new_account):
        """
            Creates an account instance.

            :param **new_account:
            :return:
        """

        cls._check_required_attributes(new_account)

        new_account_id = Client(session).post(cls._route, (), new_account)

        return Account(id=new_account_id, **new_account)

    def get_asset_by_phone(self, session, phone):
        return Asset(**self._get_contract_by_phone("assets", session, phone))

    def get_order_item_by_phone(self, session, phone):
        return OrderItem(**self._get_contract_by_phone("order-items", session, phone))

    def _get_contract_by_phone(self, entity, session, phone):
        """
            Retrieve the order_item/asset associated to a given account and filter one
            corresponding to a given phone number, from the MM sytem.

            :param: entity: string representing which object we want (order-items or asset)
                    session: MM login session instance,
                    phone: Phone number of the contract we expect
            :return: dict containing the object from the MM response
        """
        params = {
            "rowsPerPage": 1,
            "actualPage": 1,
            "phone": phone
        }

        response = Client(session).get(
            "{}/{}/{}".format(self._route, self.id, entity),
            **params)

        return response["rows"][0]

    def _check_required_attributes(new_account):
        required_attributes = ['documentNumber', 'documentType', 'email',
                               'phone', 'postalCode', 'province', 'region', 'roadName',
                               'roadNumber', 'roadType', 'town']

        if new_account.get('documentType') == "1":
            # CIF. Organization client
            required_attributes.append("corporateName")
        else:
            # VAT/NIF. Particular client
            required_attributes.extend(["name", "surname", "nationality"])

        account_filled_attributes = set(filter(
            new_account.get,
            new_account.keys()))

        missing_attributes = set(required_attributes) - account_filled_attributes

        if len(missing_attributes) != 0:
            raise AccountRequiredParamsError(sorted(missing_attributes))



from pymasmovil.client import Client
from pymasmovil.errors.exceptions import MissingLoginCredentialsError, AutenthicationError
import os


class Session():

    def __init__(self, session_id):
        self.session_id = session_id

    @classmethod
    def create(cls):

        route = '/v0/login-api'

        login_params = (
            ('username', os.getenv('MM_USER')),
            ('password', os.getenv('MM_PASSWORD')),
            ('domain', os.getenv('MM_DOMAIN')),
        )

        if login_params[0][1] is None:
            raise MissingLoginCredentialsError('MM_USER')
        elif login_params[1][1] is None:
            raise MissingLoginCredentialsError('MM_PASSWORD')
        elif login_params[2][1] is None:
            raise MissingLoginCredentialsError('MM_DOMAIN')

        session = Client().post(route=route, params=login_params, body={})

        if not session["sessionId"]:
            raise AutenthicationError

        return Session(session["sessionId"])



class MissingLoginCredentialsError(Exception):
    """Raised when login credentials are missing"""

    message = """
    Autenthication failed.
    The required configuration setting {} was not found in your environment.
    To authenticate follow the instructions explained in the README:
    https://gitlab.com/coopdevs/pymasmovil#login
    """

    def __init__(self, missing_credential):
        self.message = self.message.format(missing_credential)
        super().__init__(self.message)


class AutenthicationError(Exception):
    """Raised when login to MM API failed"""

    message = """
    Autenthication to MM API failed. We may be using wrong login
    credentials or because there is a problem in the MM server side.
    """

    def __init__(self):
        super().__init__(self.message)


class AccountRequiredParamsError(Exception):
    """Raised when trying to create an account without some required paramether"""

    message = "Missing or empty attributes required to create an account: {}"

    def __init__(self, missing_argument_list):
        self.message = self.message.format(', '.join(missing_argument_list))
        super().__init__(self.message)


class NewLineRequestRequiredParamsError(Exception):
    """Exception raised when some compulsary attributes for the portability process
    are missing in the OTRS request."""

    message = "Missing or empty attributes required for the requested {}: {}"

    def __init__(self, is_portability, missing_argument_list):
        request_type = "portability" if is_portability else "new phone number registration"

        self.message = self.message.format(request_type, ', '.join(missing_argument_list))
        super().__init__(self.message)


class UnknownMMError(Exception):
    """Raised when the MM API returns an error with an unknown structure"""

    def __init__(self, MM_response_body):
        self.message = MM_response_body
        super().__init__(self.message)



from datetime import datetime


class NewLineRequest():
    """
    Since GET /order-item/:id response and the POST /order-item request don't
    match except for a few attributes, this class aims to build an OrderItem instance
    mapping these few matching attributes.
    The specification doesn't explicitly state the optimistic equivalence presented here,
    so this mapping needs to be reviewed later on.
    """

    def __init__(self, new_order_item, account_id):
        self.lineInfo = new_order_item['lineInfo'][0]
        self.account_id = account_id

    def to_order_item(self):
        """
        Maps the atttributes of a NewLineRequest instance, as specified in Swagger,
        to the attributes of an OrderItem instance, as specified in a
        GET response.

        :return: dict
        """

        return {
            'account_id': self.account_id,
            'name': self.lineInfo.get('name'),
            'surname': self.lineInfo.get('surname'),
            'phone': self.lineInfo.get('phoneNumber'),
            'orderType': 'OrderItem',
            'status': 'Esperando para enviar',
            'productName': 'Unknown',
            'createdDate': datetime.now(),
            'lastModifiedDate': datetime.now(),
            'attributes': {
                'Nombre': self.lineInfo.get('name'),
                'Apellidos': self.lineInfo.get('surname'),
                'Fecha_Portabilidad_Saliente': datetime.strptime(
                    self.lineInfo.get('portabilityDate', "2020-09-03"), "%Y-%m-%d"),
                'ICCID_Donante': self.lineInfo.get('iccid_donante'),
                'Tipo_de_Documento': self.lineInfo.get('documentType'),
                'Numero_de_Documento': self.lineInfo.get('docid'),
                'Operador_Donante_Movil': self.lineInfo.get('donorOperator'),
                'Operador_Receptor_Movil': 'Som Connexió',
                'Fecha_de_solicitud_del_abonado': datetime.now(),
            },
            'simAttributes': {
                'ICCID': self.lineInfo.get('iccid'),
            }
        }



from pymasmovil.client import Client
from pymasmovil.models.new_line_request import NewLineRequest
from pymasmovil.models.contract import Contract
from pymasmovil.errors.exceptions import NewLineRequestRequiredParamsError


class OrderItem(Contract):

    _route = '/v0/order-items'
    orderType = ''
    lastModifiedDate = ''

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @classmethod
    def get(cls, session, order_item_id):
        contract = super().get(session, order_item_id)

        return cls(**contract['orderDetail'])

    @classmethod
    def create(cls, session, account, **new_order_item):
        """
            Creates an order-item and posts it to the given account

            :param account: account where we want to add the order-item
            :param **new_order_item:
            :return: order-item instance
        """

        post_route = '/v0/accounts/{}/order-items'.format(account.id)

        cls._check_required_attributes(new_order_item)

        Client(session).post(post_route, (), new_order_item)

        new_line_request = NewLineRequest(new_order_item, account.id)

        return OrderItem(**new_line_request.to_order_item())

    def _check_required_attributes(new_order_item):
        """ Check that all compulsary attributes for a portability request
        are present in the request dynamic fields """

        required_attributes = ['idLine', 'documentType', 'iccid', 'productInfo']

        is_portability = False
        new_line_info = new_order_item['lineInfo'][0]

        if new_line_info.get('phoneNumber'):
            is_portability = True
            required_attributes.extend(["docid", "donorOperator", "portabilityDate"])

        if new_line_info.get('documentType') == "CIF":
            # Organization client
            required_attributes.append("corporateName")

        elif new_line_info.get('documentType') in ("NIF", "NIE", "Pasaporte"):
            # Particular client
            required_attributes.extend(["name", "surname"])

        order_item_filled_attributes = set(filter(
            new_line_info.get,
            new_line_info.keys()))

        missing_attributes = set(required_attributes) - order_item_filled_attributes

        if len(missing_attributes) != 0:
            raise NewLineRequestRequiredParamsError(is_portability, sorted(missing_attributes))



from pymasmovil.models.contract import Contract


class Asset(Contract):
    _route = '/v1/assets'

    maxNumTariff = ''
    numTariff = ''
    productRelation = ''
    assetType = ''
    initDate = ''

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @classmethod
    def get(cls, session, asset_id):
        contract = super().get(session, asset_id)

        return cls(**contract['asset'])



import os
import requests
import json
from pymasmovil.errors.http_error import HTTPError
from pymasmovil.errors.mm_error import MMError
from pymasmovil.errors.exceptions import MissingLoginCredentialsError


class Client(object):

    def __init__(self, session=None):

        self.base_url = os.getenv('MM_BASEURL')

        if self.base_url is None:
            raise MissingLoginCredentialsError('MM_BASEURL')

        self.headers = {
            "Accept": "application/json",
            "Content-Type": "application/json",
        }
        if session:
            self.session = session
            self.headers["Authorization"] = "Bearer {}".format(self.session.session_id)

    def get(self, route, **kwargs):
        """ Sends a GET HTTP request

        Args:
            route (str): String with the route to the endpoint

        Return:
            **response**: Returns the response object
        """
        return self._send_request(
            verb="GET",
            url="{url}{route}".format(url=self.base_url, route=route),
            params=kwargs,
        ).json()

    def post(self, route, params, body):
        """ Sends a POST HTTP request

        Args:
            route (str): String with the route to the endpoint
            params (tuple): Tuple with optional parameters of the request to send
            body (dict): Dict with the body of the request to send

        Return:
            **response**: Returns the response object
        """

        return self._send_request(
            verb="POST",
            url="{url}{route}".format(url=self.base_url, route=route),
            params=params,
            payload=body,
        ).json()

    def _send_request(self, verb, url, payload=None, params={}):
        """send the API request using the *requests.request* method

        Args:
            payload (dict), params (dict)

        Returns:
            **requests.Response**: Response received after sending the request.

        .. note::
            Supported HTTP Methods: GET, POST
        """

        json_payload = None
        if payload:
            json_payload = json.dumps(payload)

        response = requests.request(verb.upper(),
                                    url,
                                    headers=self.headers,
                                    data=json_payload,
                                    params=params)

        if response.status_code not in range(200, 300):
            error = MMError()
            error.build(response)
            raise HTTPError(error)

        return response



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

with open('README.md') as f:
    README = f.read()


VERSION = '0.0.8'

setup(
    name='pymasmovil',
    version=VERSION,
    author='Coopdevs',
    author_email='gerard.funosas@somconnexio.coop',
    maintainer='Daniel Palomar, Pau Pérez, Gerard Funosas',
    url='https://gitlab.com/coopdevs/pymasmovil',
    description="Python client for the Más Móvil's B2B API",
    long_description=README,
    long_description_content_type='text/markdown',
    packages=find_packages(exclude=('tests', 'docs')),
    include_package_data=True,
    zip_safe=False,
    install_requires=['requests'],
    test_suite='unittest2.collector',
    tests_require=['unittest2', 'mock', 'tox', 'coverage', 'httpretty'],
)
